// Pawe Pasznik H1ISI //
//---------------------//

/*
	Algorytm find_first_of zaimplementowany jest jako dwie przecione funkcje:

	template <class InputIterator, class ForwardIterator>
		InputIterator find_first_of(InputIterator first1, InputIterator last1,
                            ForwardIterator first2, ForwardIterator last2);

	template <class InputIterator, class ForwardIterator, class BinaryPredicate>
		InputIterator find_first_of(InputIterator first1, InputIterator last1,
                            ForwardIterator first2, ForwardIterator last2,
                            BinaryPredicate comp);

	InputIterator - to taki iterator z ktrego moe by wyuskany obiekt, powinien te pozwala wskazywa na nastpny iterator w sekwencji.

	ForwardIterator - to taki iterator, ktry suy do przegldania liniowej sekwencji wartoci, aczkolwiek nie potrafi dziaa wstecz jak sama nazwa wskazuje. Doskonale nadaje si do wieloprzejciowego przegldania danych.

	Funkcje te rni si tym, e w pierwszej domylnym predykatem jest po prostu operator== (musi istnie operator== dla argumentw typu
	wyuskanych InputIterator i ForwardIterator), a w drugim mona poda wasny warunek za pomoc predykatu lub funkcji.
	
	find_first_of operuje na dwch kontenerach, przegldajac je za pomoc iteratorw. Dla rozrnienia nazwe je konternerem wzorcowym i kontenerem przesukiwanym.

	find_first_of(przeszukiwanyKontener.begin(), przeszukiwanyKontener.end(), kontener_wzorcowy.begin(), kontener_wzorcowy.end());

	Dziaa podobnie jak zwyky find z t rnic, e wyszukuje w zakresie wyznaczonym iteratorami kontenera wzorcowego pierwszego wystpienia wartoci
	w kontenerze przeszukiwanym (rwnie w jego czci lub caoci wyznaczonej poprzez iteratory. 

	Aby skorzysta z algorytmu find_first_of naley wczy plik nagwkowy: algorithm

*/

#include <algorithm>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <list>
#include <iostream>

/*
	Przykadowym zastosowaniem find_first_of jest wyszukiwanie biaych znakw w tekcie, np. aby zliczy sowa.
*/

int countWords(const char* str)
{
	const char* whitespace = "\t\n \0";	//znaki traktowane jako biae
	const int numberOfWhitespaces = 4;	//nie moge si posuy strlen poniewa zaley mi, eby znak \0 rwnie si zawiera
	unsigned words = 0;
	
	while (*str != '\0')			//kady cig znakowy w stylu C koczy si znakiem \0, jest on te wyszukiwany w zbiorze
	{					//biaych znakw, aby uwzgldni ostatnie sowo w obiczeniach
		++str; 				//przeskakuje do badania kolejnego znaku (dopiero tutaj aby nie wyj poza zakres cigu znakowego.
		++words;			//jeli nadal jestem w ptli doliczam znalezienie sowa

		//zmieniam pocztek przeszukiwanego cigu napisowego na pierwszy znalezioniony biay znak
		//(cig przesukiwany: cay znak od ostatniego znalezionego biaego znaku lub pocztku sowa; cig wzorcowy: biae znaki)
		str = std::find_first_of(str, str + strlen(str), whitespace, whitespace + numberOfWhitespaces);	
	}
return words;
}

/*
	Warto zauway, e kontener std::string zawiera metod find_first_of dziaajc troch inaczej.
	(nie mona jej myli z wyej przedstawionym algorytmem, nawet nie potrzebuje pliku nagwkowego: algorithm)

	size_t find_first_of ( const string& str, size_t pos = 0 ) const;
	size_t find_first_of ( const char* s, size_t pos, size_t n ) const;
	size_t find_first_of ( const char* s, size_t pos = 0 ) const;
	size_t find_first_of ( char c, size_t pos = 0 ) const;
	
	Nie operuje na iteratorach. Zwraca pozycje w napisie. Cigiem przeszukiwanym jest tutaj obiekt na rzecz, ktrego metoda jest wywoywana,
	a cigiem wzorcowym podany jako argument cig znakw (ew. znak lub std::string)

	Argumenty:
	s / str / c - wyszukiwane znaki
	pos - pozycja w obiekcie stringa na rzecz ktrego metoda jest wywoywana, od ktrej ma by zaczte wyszukiwanie.
	n - ilo znakw w s, ktre maj by brane pod uwag podczas wyszukiwania.
*/

size_t firstColor(const std::string str, size_t intro)	//intro: ile znakw na od pocztku std::string ma pomin.
{
	char colors[4] = "RGB";				//cig znakowy wyszukiwanych znakw (w przykadzie kolory)
	return str.find_first_of(colors, intro);
}


/*
	Bardziej wyrafinowanym przykadem uycia algorytmu find_first_of jest zastosowanie go z innym typem kontenera ni napisowy,
	z wasnym predykatem lub przecionym operatorem operator==
	Poniej zaprezentuje przykad z przecionym operatorem operator==.
*/

class Owoc
{
	std::string nazwa_;
	unsigned nr;
public:
	Owoc(std::string nazwa) : nazwa_(nazwa) {static unsigned last = 0; nr = last++;}	//wprowadzam numeracje w konstruktorze, unikam w ten sposb powtrze
	bool operator==(const Owoc &owoc) const {return owoc.nr == nr;}				//elementem rozrniajcym owoce jest ich numer, a nie nazwa.
	bool operator!=(const Owoc &owoc) const {return !(operator==(owoc));}
	friend std::ostream& operator<<(std::ostream &o, const Owoc &owoc);
};

std::ostream& operator<<(std::ostream &o, const Owoc &owoc)
{
	o << owoc.nazwa_ << " nr " << owoc.nr;
	return o;
}

void przyklad3()
{
	std::list<Owoc> posiadane;
	std::list<Owoc> smaczne;

	//Tworze owoce
	Owoc A("Jablko");		//nr == 0
	Owoc B("Gruszka");		//1
	Owoc C("Morela");		//2
	Owoc D("Sliwka");		//3 dwie rne liwki - rozrniane po numerze.
	Owoc E("Sliwka");		//4
	Owoc F("Kiwi");			//5

	//Kopie tych owocw umieszczam w posiadanych.
	posiadane.push_back(A);
	posiadane.push_back(B);
	posiadane.push_back(C);
	posiadane.push_back(D);
	posiadane.push_back(E);
	posiadane.push_back(F);

	smaczne.push_back(B);
	smaczne.push_back(C);
	smaczne.push_back(E);		//tylko jedna liwka naley do smacznych

	std::list<Owoc>::iterator iter;
	do
	{
		iter = std::find_first_of(posiadane.begin(), posiadane.end(), smaczne.begin(), smaczne.end());
		if (iter != posiadane.end())
		{
			std::cout << "Zjedzono smaczny owoc: " << *iter << std::endl;
			posiadane.erase(iter);			//zjadam wic usuwam z listy posiadanych
		}
	} while (iter != posiadane.end());
}

int main()
{
//Przykad 1
	std::cout << "Count words: " << countWords("Dlugi napis testowy z duza iloscia znakow.") << std::endl;

//Przykad 2
	std::string ZapisKolorow = "Zapis Kolorw Grafiki: R114G111B103";
	int pos = firstColor(ZapisKolorow,0);			//wykonuje wyszukiwanie na caym stringu, efektem jest zbdna interpretacja napisu wprowadzajcego jako kolor
	std::cout << "Kolor " << ZapisKolorow[pos] << ": " << ZapisKolorow[pos + 1] << ZapisKolorow[pos + 2] << ZapisKolorow[pos + 3] << std::endl;
	pos = firstColor(ZapisKolorow,23);			//pomijam napis wprowadzajcy
	std::cout << "Kolor " << ZapisKolorow[pos] << ": " << ZapisKolorow[pos + 1] << ZapisKolorow[pos + 2] << ZapisKolorow[pos + 3] << std::endl;

//Przykad 3
	przyklad3();
	return 0;
}
